/* File Name: PrefsWindow.cpp */

/*
************************************************************************
************************************************************************

Copyright © 1998 Multimedia Integration Products, All Rights Reserved

This file is not for public release.
************************************************************************

************************************************************************
************************************************************************ */


#include "PrefsWindow.h"

#include "main.h"

// *********************************************//************
PrefsWindow::PrefsWindow()
	: BWindow(BRect(100, 100, 545, 455), "BeVC Preferences", B_TITLED_WINDOW, B_NOT_ZOOMABLE )
{
	workDirFilePanel = NULL;
	repositoryFilePanel = NULL;
	
	tempWorkDirPath = workDirPath;
	tempRepositoryPath = repositoryPath;
	
	confirmClose = true;						// See description in PrefsWindow.h
	
	LayoutWindow();
}


// *********************************************//************
PrefsWindow::~PrefsWindow()
{
	
}

// *********************************************//************
void PrefsWindow::MessageReceived(BMessage * message)
{
	switch (message->what)
	{
		
		case BVC_PREFS_PICK_REPOSITORY:
		{
			MakeFilePanel("Choose Repository", BVC_PREFS_REPOSITORY_CHOSEN, repositoryFilePanel);

		}
		break;
		
		case BVC_PREFS_REPOSITORY_CHOSEN:
		{

			if (message->FindFlat("refs", &tempRepositoryPath) != B_OK)
				return;
			
			repositoryView->SetText(tempRepositoryPath.Path());
			
		}
		break;
		
		case BVC_PREFS_PICK_WORK_DIR:
		{
			MakeFilePanel("Choose Work Directory", BVC_PREFS_WORK_DIR_CHOSEN, workDirFilePanel);
			
		}
		break;
		
		case BVC_PREFS_WORK_DIR_CHOSEN:
		{

			if (message->FindFlat("refs", &tempWorkDirPath) != B_OK)
				return;
			
			workDirView->SetText(tempWorkDirPath.Path());
			
		}
		break;
		
		case BVC_PREFS_OK:
		{
			SavePrefs();
			confirmClose = false;
			this->PostMessage(B_QUIT_REQUESTED);
		}
		break;
		
		case BVC_PREFS_CANCEL:
		{
			confirmClose = false;
			this->PostMessage(B_QUIT_REQUESTED);
		}
		break;
		
		case BVC_PREFS_LOCAL_REPOS:
		{
			pickReposButton->SetEnabled(true);
			repositoryView->SetHighColor(0, 0, 0, 255);
			repositoryView->Invalidate();
			remoteRepositoryView->SetEnabled(false);
		}
		break;
		
		case BVC_PREFS_REMOTE_REPOS:
		{
			pickReposButton->SetEnabled(false);
			repositoryView->SetHighColor(170, 170, 170, 255);
			repositoryView->Invalidate();
			remoteRepositoryView->SetEnabled(true);
		}
		break;
		
		default:
			inherited::MessageReceived(message);
		break;
		
	}
}


// *********************************************//************
bool PrefsWindow::QuitRequested()
{
	if (confirmClose)
	{
		int32 result;
		BAlert	* alert;
		
		alert = new BAlert("BeVC", "Save preferences before closing?", 
							"Cancel", "No", "Yes", 
							 B_WIDTH_AS_USUAL, B_WARNING_ALERT); 
		alert->SetShortcut(0, B_ESCAPE); 
		alert->SetShortcut(1, 'n'); 
		alert->SetShortcut(2, 'y'); 
		
		result = alert->Go();
		
		switch (result)
		{
			case 0:							// Cancel
				confirmClose = true;
				return false;
			break;
			
			case 1:							// No
				return true;
			break;
			
			case 2:							// Yes
				confirmClose = false;
				SavePrefs();
				return true;
				
			default:						// huh?
				return false;
		}
	}
	
	be_app->PostMessage(VC_RELOAD_MODULE_LIST);
	return true;
}


// *********************************************//************
void PrefsWindow::MakeFilePanel(const char * name, uint32 command, BFilePanel * which)
{
	if (which == NULL)
	{
		which = new BFilePanel(B_OPEN_PANEL, new BMessenger(this), NULL, B_DIRECTORY_NODE,
								false, new BMessage(command), NULL, false, true);
	}
	which->Show();

}


// *********************************************//************
void PrefsWindow::LayoutWindow()
{
	winView = new BView(Bounds(), "winView", B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
	winView->SetViewColor(225, 225, 225, 255);
	
	userNameView = new BTextControl(BRect(20, 20, 250, 40), "userName", "User Name:", userName, NULL);
	userNameView->SetDivider(60);
	winView->AddChild(userNameView);
	
	passwordView = new BTextControl(BRect(20, 50, 250, 95), "password", "Password:", password, NULL);
	passwordView->SetDivider(60);
	winView->AddChild(passwordView);
	
	BStringView	* stringView;
	BButton		* button;
	
	
/* Work Dir */
	
	workDirBox = new BBox(BRect(20, 90, Bounds().right - 20, 180), "workDirBox", B_FOLLOW_LEFT_RIGHT);
	workDirBox->SetLabel("Work Directory");
	AddChild(workDirBox);

	button = new BButton(BRect(15, 25, 90, 50), "pickWorkDir", "Choose", new BMessage(BVC_PREFS_PICK_WORK_DIR));
	workDirBox->AddChild(button);
	
	BRect bounds(workDirBox->Bounds());
	bounds.left = 105;
	bounds.top = 35;
	bounds.bottom = 45;
	bounds.right -= 15;
	workDirView = new BStringView(bounds, "workDirView", tempWorkDirPath.Path(), B_FOLLOW_LEFT_RIGHT);
	workDirBox->AddChild(workDirView);
	
	bounds = workDirBox->Bounds();
	bounds.left += 15;
	bounds.top = bounds.bottom - 30;
	bounds.bottom -= 10;
	bounds.right = 140;
	deleteAfterPrefBox = new BCheckBox(bounds, "deleteAfterPrefBox", "Empty After Checkin", NULL);
	deleteAfterPrefBox->SetValue(deleteAfterCheckin);
	workDirBox->AddChild(deleteAfterPrefBox);
	
	
/* Pick Repository */
	repositoryBox = new BBox(BRect(20, 195, Bounds().right - 20, 295), "repositoryBox", B_FOLLOW_LEFT_RIGHT);
	repositoryBox->SetLabel("Repository");
	winView->AddChild(repositoryBox);
	
	localReposRadio = new BRadioButton(BRect(15, 25, 80, 50), "localReposRadio", "Local", new BMessage(BVC_PREFS_LOCAL_REPOS));
	repositoryBox->AddChild(localReposRadio);
	
	remoteReposRadio = new BRadioButton(BRect(15, 60, 80, 85), "remoteReposRadio", "Remote", new BMessage(BVC_PREFS_REMOTE_REPOS));
	repositoryBox->AddChild(remoteReposRadio);
	
	remoteRepositoryView = new BTextControl(BRect(95, 60, 300, 80), "remoteRepositoryView", "BeVC Server:", serverAddress, NULL);
	remoteRepositoryView->SetDivider(70);
	repositoryBox->AddChild(remoteRepositoryView);
	
	pickReposButton = new BButton(BRect(90, 21, 165, 46), "pickRepository", "Choose", new BMessage(BVC_PREFS_PICK_REPOSITORY));
	repositoryBox->AddChild(pickReposButton);
	
	bounds = repositoryBox->Bounds();
	bounds.left = 180;
	bounds.top = 31;
	bounds.bottom = 41;
	bounds.right -= 15;
	repositoryView = new BStringView(bounds, "repositoryView", tempRepositoryPath.Path(), B_FOLLOW_LEFT_RIGHT);
	repositoryBox->AddChild(repositoryView);
	
	if(remoteRepository)
	{
		remoteReposRadio->SetValue(true);
		pickReposButton->SetEnabled(false);
		repositoryView->SetHighColor(200, 200, 200, 255);
	}
	else
	{
		localReposRadio->SetValue(true);
		remoteRepositoryView->SetEnabled(false);
	}
	
/* OK and Cancel Buttons */
	bounds = winView->Bounds();
	bounds.top = bounds.bottom - 40;
	bounds.left = bounds.right - 95;
	bounds.right -= 20;
	bounds.bottom -= 15;
	button = new BButton(bounds, "okButton", "OK", new BMessage(BVC_PREFS_OK), B_FOLLOW_RIGHT);
	winView->AddChild(button);
	button->MakeDefault(true);
	
	bounds = winView->Bounds();
	bounds.top = bounds.bottom - 40;
	bounds.left = bounds.right - 185;
	bounds.right = bounds.right - 110;
	bounds.bottom -= 15;
	button = new BButton(bounds, "cancelButton", "Cancel", new BMessage(BVC_PREFS_CANCEL), B_FOLLOW_RIGHT);
	winView->AddChild(button);
	
	AddChild(winView);
	
	float minWidth, maxWidth, minHeight, maxHeight;
	GetSizeLimits(&minWidth, &maxWidth, &minHeight, &maxHeight);
	SetSizeLimits(445, maxWidth, 355, 355);
}

// *********************************************//************
void PrefsWindow::SavePrefs()
{
	strcpy(userName, userNameView->Text());
	strcpy(password, passwordView->Text());
	
	workDirPath = tempWorkDirPath;
	deleteAfterCheckin = (deleteAfterPrefBox->Value() == B_CONTROL_ON);
	
	remoteRepository = (remoteReposRadio->Value() == B_CONTROL_ON);
	repositoryPath = tempRepositoryPath;
	strcpy(serverAddress, remoteRepositoryView->Text());
}