/* File Name: SourceControlWindow.cpp */

/*
************************************************************************
************************************************************************

Copyright © 1998 Multimedia Integration Products, All Rights Reserved

This file is for public release.

This file is for reference use only. Any redistribution must include
the entire BeVC package, including source code. If you use this code
please notify j-onorato@uchicago.edu or jwhamil@students.wisc.edu

************************************************************************

************************************************************************
************************************************************************ */


#include "SourceControlWindow.h"
#include "ModuleItem.h"
#include "main.h"
#include "NetMessenger.h"

#include <string.h>

// *********************************************//************
SourceControlWindow::SourceControlWindow()
	: BWindow(BRect(20, 50, 628, 300), "Be Version Control", B_DOCUMENT_WINDOW, 0 )
{
	prefsWindow = NULL;
	
	SetupMenus();
	SetupListView();
	
	GetModulesFromRepos();
	
	Show();
}


// *********************************************//************
SourceControlWindow::~SourceControlWindow()
{

}


// *********************************************//************
bool SourceControlWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(true);
}

void SourceControlWindow::MessageReceived(BMessage * message)
{
	switch (message->what)
	{
		case VC_MENU_FILE_PREFS:
		{
			prefsWindow = new PrefsWindow();
			prefsWindow->Show();
		}
		break;
		
		case VC_RELOAD_MODULE_LIST:
		{
			GetModulesFromRepos();
		}
		break;
		
		case VC_MENU_FILE_QUIT:
		{
			be_app->PostMessage(B_QUIT_REQUESTED);
		}
		break;
		
		case VC_MENU_MODULE_UPDATE:
		{
			BMessage request(MIP_VC_UPDATE);
			request.AddString("user", userName);
			request.AddString("passwd", password);
			SendToRepos(&request);			//Ask for the modules this user doesn't have
		}
		break;
		
		case MIP_VC_UPDATE_REPLY:
		{
			Update(message);
		}
		break;
		
		case VC_MENU_MODULE_CHECKOUT:
		{
			CheckOut();
		}
		break;
		
		case VC_MENU_MODULE_CHECKIN:
		{
			BMessage	request(MIP_VC_MODS_OUT_LIST);
			request.AddString("user", userName);
			request.AddString("passwd", password);
			SendToRepos(&request); //Ask for the modules out for this user
		}
		break;
		
		case VC_MENU_MODULE_ADD:
		{
			
		}
		break;
		
		case VC_MENU_MODULE_DELETE:
		{
			
		}
		break;

		case B_REFS_RECEIVED:
		{
			beep();
		}
		break;
		
		case MIP_VC_MODS_OUT_LIST_REPLY:
		case MIP_VC_CHECK_IN_REPLY:
		{
			CheckIn(message);
		}
		break;
		
		case MIP_VC_CHECK_OUT_REPLY:
		{
			UnZipModule(message, workDirPath.Path());
		}
		break;
		
		case MIP_VC_MODS_LIST:
		{
			LoadModuleList(message);
		}
		break;
		
		case MIP_VC_USER_PASS_ERROR:
		{
			BAlert	* alert;
			
			alert = new BAlert("Be Version Control", "Please enter a username and password in the Preferences window.", 
								"OK", NULL, NULL, 
								 B_WIDTH_AS_USUAL, B_WARNING_ALERT); 
			
			alert->Go();

		}
		break;
		
		case MIP_VC_AUTH_FAILED:
		{
			BAlert	* alert;
			
			alert = new BAlert("Be Version Control", "Invalid name or password. Please check the Preferences window.", 
								"OK", NULL, NULL, 
								 B_WIDTH_AS_USUAL, B_WARNING_ALERT); 
			
			alert->Go();
			
		}
		break;
		
		default:
			inherited::MessageReceived(message);
		break;
		
	}
}


// *********************************************//************
void SourceControlWindow::SetupMenus()
{
	BMenuBar	* menuBar;
	BMenu		* menu;
	BMenuItem	* item;
	BMessage	* message;
	
	BRect		menuBarRect(Bounds());

	menuBar = new BMenuBar(menuBarRect, "menuBar");
	
/* File menu */
	menu = new BMenu("File");
	
	message = new BMessage(VC_MENU_FILE_PREFS);
	item = new BMenuItem("Preferences…", message, ';');
	menu->AddItem(item);
	
	menu->AddSeparatorItem();

	message = new BMessage(VC_SHOW_CHANGE_PASSWD_WINDOW);
	item = new BMenuItem("Administer Local Passwords…", message);
	item->SetTarget(be_app);
	menu->AddItem(item);


	menu->AddSeparatorItem();

	message = new BMessage(VC_MENU_FILE_QUIT);
	item = new BMenuItem("Quit", message, 'Q');
	menu->AddItem(item);
	
	menuBar->AddItem(menu);
	
/* Module menu */
	menu = new BMenu("Module");
	
	message = new BMessage(VC_MENU_MODULE_CHECKOUT);
	item = new BMenuItem("Check Out", message, 'O');
	menu->AddItem(item);

	message = new BMessage(VC_MENU_MODULE_UPDATE);
	item = new BMenuItem("Update", message, 'U');
	menu->AddItem(item);

	message = new BMessage(VC_MENU_MODULE_CHECKIN);
	item = new BMenuItem("Check In", message, 'I');
	menu->AddItem(item);
	
	menu->AddSeparatorItem();

	message = new BMessage(VC_RELOAD_MODULE_LIST);
	item = new BMenuItem("Reload Module List", message, 'R');
	menu->AddItem(item);

	menu->AddSeparatorItem();

	message = new BMessage(VC_MENU_MODULE_ADD);
	item = new BMenuItem("Add…", message);
	item->SetEnabled(false);
	menu->AddItem(item);

	message = new BMessage(VC_MENU_MODULE_DELETE);
	item = new BMenuItem("Delete", message);
	item->SetEnabled(false);
	menu->AddItem(item);
	
	menuBar->AddItem(menu);
	
	
	
/* Add the menuBar to this window */
	AddChild(menuBar);
}


// *********************************************//************
void SourceControlWindow::SetupListView()
{
	BScrollView		* scrollView;
	CLVColumn		* column;
	
/* Calculate where the column list goes */
	BRect frame(Bounds());
	frame.top = 19;
	frame.bottom -= 14;
	frame.right -= 14;

/* Create the column list view */
	moduleList = new ColumnListView(frame, &scrollView, "moduleList",
										B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE,
										B_MULTIPLE_SELECTION_LIST);
	
/* Make the columns */
	column = new CLVColumn("Ver.", 50, 0, 20);
	moduleList->AddColumn(column);
	
	column = new CLVColumn("Name", 200, 0, 20);
	moduleList->AddColumn(column);
	
	column = new CLVColumn("Status", 80, 0, 20);
	moduleList->AddColumn(column);
	
	column = new CLVColumn("Who", 90, 0, 20);
	moduleList->AddColumn(column);
	
	column = new CLVColumn("Time", 50, 0, 140);
	moduleList->AddColumn(column);
	
/* Add the scroll view to the window */
	AddChild(scrollView);

}


// *********************************************//************
void SourceControlWindow::LoadModuleList(BMessage *modList)
{
	status_t	err;
	int32		numMods;	
	type_code	typeCode;
												//How many are in there?
	modList->GetInfo("module", &typeCode, &numMods);
	
	Lock();
		moduleList->MakeEmpty();				//Clean up from before
	Unlock();
	
	for (int i=0; i<numMods; i++)
	{
		char	*verNoTemp;
		char	*statusTemp;
		char	*userNameTemp;
		char	*nameTemp;
		char	*timeTemp;
		
		BMessage	modInfo;
		if (!modInfo.IsEmpty())					//Make sure it's clean
			modInfo.MakeEmpty();
		
												//Find the message for this module
		err = modList->FindMessage("module", i, &modInfo);

/* Get the module info */
		modInfo.FindString("ver", &verNoTemp);
		modInfo.FindString("name", &nameTemp);
		modInfo.FindString("status", &statusTemp);
		modInfo.FindString("who", &userNameTemp);
		modInfo.FindString("time", &timeTemp);
		
/* Add the module */
		Lock();	
			ModuleItem *modItem = new ModuleItem(verNoTemp, nameTemp, statusTemp, userNameTemp, timeTemp);
			moduleList->AddItem(modItem);		//Put this module in the list
		Unlock();
	}
}


// ********************************************//************
status_t SourceControlWindow::CheckIn(BMessage * msg)
{
/*
   Then it deletes everything in the work dir. - Note - YOU CAN'T
   KEEP STUFF IN THE WORK DIR - according to the pref.
*/
	BMessage	mods(MIP_VC_CHECK_IN);
	int32		numMods;
	status_t	err;
	type_code	typeCode;
	
												//How many are in there?
	err = msg->GetInfo("module", &typeCode, &numMods);

	for (int i = 0; i<numMods; i++)
	{
		BPath	curModPath(workDirPath);
		char	*modName;
		
		err = msg->FindString("module", i, &modName);
		if (err != B_OK)
			return err;
		err = curModPath.Append(modName);
		if (err != B_OK)
			return err;
		
												//Zip the mod and add to the message
		err = ZipModule(curModPath.Path(), &mods);
		if (err != B_OK)
			return err;
	}
	
	printf("This is going to the repos as mods in\n");
	mods.PrintToStream();
	
	err = mods.AddString("user", userName);
	err = mods.AddString("passwd", password);

	err = SendToRepos(&mods);
	if (err != B_OK)
		return err;
		
/* Kill everything in the work */
/* This should really wait for a confirmation before killing the work directory */
	if(deleteAfterCheckin)
	{
		char	theCall[1024];
		sprintf(theCall, "rm -R %s", workDirPath.Path());		
		system(theCall);

		err = create_directory(workDirPath.Path(), 0777);	
		if (err != B_OK)
			return err;
	}
	
	return B_OK;
}



// ********************************************//************
status_t SourceControlWindow::CheckOut()
{
	BMessage modRequest (MIP_VC_CHECK_OUT);
	modRequest.AddString("user", userName);
	modRequest.AddString("passwd", password);
	
	ModuleItem * module;
	int i = 0;
	int selected;
	
	
	while ((selected = moduleList->CurrentSelection(i)) >= 0)
	{
		module = (ModuleItem *) moduleList->ItemAt(selected);
		if (module != NULL)
		{	
			modRequest.AddString("module", module->nameStr);
		}
		i++;
	}
	
	SendToRepos(&modRequest);
	
	
	if (i != 0)									// If there were any selected mods
	{
		GetModulesFromRepos();
		return B_OK;
	}
	else
		return B_BAD_VALUE;						// There were no mods selected
}


// ********************************************//************
status_t SourceControlWindow::UnZipModule(BMessage * msg, const char * destination)
// Takes a BMessage with modules in the data field 'module' and puts them
// in a temp file then unzips the files into the destination folder
{
	status_t	err;
	int32		numMods;
	type_code	typeCode;
												//How many are in there?
	err = msg->GetInfo("module", &typeCode, &numMods);
	
//	printf("There are %i module's in the checkout message\n", numMods);
	
	for (int i=0; i<numMods; i++)
	{
		char	* name;
		char	* buf;
		ssize_t	size;
		
//		msg->PrintToStream();
		msg->FindString("name", i, &name);
		msg->FindData("module", B_ANY_TYPE, i, &buf, &size);
		
		err = MakeFileFromBuffer(buf, "/tmp/MIP_BVC_Temp.zip", size);
	
/* Unzip the files */
		char	placeToGo[B_PATH_NAME_LENGTH];
		
		strcpy (placeToGo, destination);
		strcat (placeToGo, "/");
		strcat (placeToGo, name);

		char * zipExt = strstr(placeToGo, ".zip");
		if (zipExt != NULL)
		{
			int endIndex = zipExt - placeToGo;
			placeToGo[endIndex] = '\0';
		}
		
		char	theCall[1024];
		sprintf(theCall, "/boot/beos/bin/unzip -n /tmp/MIP_BVC_Temp.zip -d %s", destination);		
		system(theCall);

		BEntry tempFile("/tmp/MIP_BVC_Temp.zip");
		tempFile.Remove();
	}	
}


// ********************************************//************
status_t SourceControlWindow::ZipModule(const char * path, BMessage * msg)
// Take the module dir at path and zip it into a temp file then
// add it to msg under the Data feild 'module'.
{
	status_t	err;
	BEntry 		tempFile;
	BPath		thePath(path);
	BPath		tempPath;
	char		blah[B_PATH_NAME_LENGTH];
	
	thePath.GetParent(&tempPath);
	
	err = tempPath.InitCheck();
	if (err != B_OK)
		return err;
	
	char	theCall[1024];
	sprintf(theCall, "cd %s; zip -r /tmp/MIP_BVC_Temp.zip %s", tempPath.Path(), thePath.Leaf());
	system(theCall);
	
	BPath		tempFilePath("/tmp/MIP_BVC_Temp.zip");
	
												// Put the file into the message
	err = FileToStream(&tempFilePath, msg, "module", "name", thePath.Leaf());
	
	err = tempFile.SetTo("/tmp/MIP_BVC_Temp.zip");
	err = tempFile.Remove();					// Kill the temp file
	
	return err;
}


// ********************************************//************
// Send this message to the repository, be it local or remote
status_t SourceControlWindow::SendToRepos(BMessage *msg)
{
	if (remoteRepository)
	{
		netstatus_t	err;
	
		status_t	status;
		try
		{
			NetMessenger msgr(serverAddress, kMIME, -1, &status, 5e-6);
			if (status != B_OK) throw (msgr.Error());
			status = msgr.SendMessage(msg);
			if (status != B_OK) throw (msgr.Error());
			return status;
		}
		catch (netstatus_t netstat)
		{
			char inform[255]; 
			switch ( netstat ) 
			{ 
				case NET_UNKNOWN_HOST: 
					sprintf(inform, "Could not resolve hostname \"%s\".", serverAddress); 
					break; 

				case NET_TIMED_OUT: 
					sprintf(inform, "The connection timed out."); 
					break; 

				case NET_CANT_CONNECT: 
					sprintf(inform, "The connection request was rejected."); 
					break; 
                                                
				case NET_INTERNAL_ERROR: 
					sprintf(inform, "A message_daemon internal error occurred. Try restarting it."); 
					break; 
                        
				case NET_NO_DAEMON: 
					sprintf(inform, "message_daemon must be running to use remote repositories."); 
					break; 
                
				case NET_DISCONNECTED: 
					sprintf(inform, "The connection was disrupted."); 
					break; 
                        
				default: 
				sprintf(inform, "An unknown error occurred."); 
				break; 
			} // case
			BAlert	* alert;
			alert = new BAlert("Be Version Control", inform, 
								"OK", NULL, NULL, 
								 B_WIDTH_AS_USUAL, B_WARNING_ALERT); 
			alert->Go();


		} // catch
	} // if
	else
		return be_app->PostMessage(msg);
}


// ********************************************//************
status_t SourceControlWindow::Update(BMessage * msg)
{
												// Delete the old one

	
												// Unzip the module
	UnZipModule(msg, workDirPath.Path());
}

// ********************************************//************
void SourceControlWindow::GetModulesFromRepos()
{
	BMessage modList(MIP_VC_GET_MODULES);
	modList.AddString("user", userName);
	modList.AddString("passwd", password);
	SendToRepos (&modList);
}